package	jp.co.sds_corp.dao.tool;

import java.io.File;
import java.io.FileWriter;

import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.IConfiguration;

public class DaoGenerator
{
	String	srcdir;
	String	destdir;
	Template	template;
	String	daoClass;
	String	dtoClass;
	TableInfo	table;
	AccessInfo		access;

	public void setSrcdir( String var ) {
		this.srcdir = var;
	}

	public void setDestdir( String var ) {
		this.destdir = var;
	}

	public void setTemplate( Template var ) {
		this.template = var;
	}

	public void generate( File file ) throws Exception {
System.out.println( "generate:" + file );

		DOMConfiguration config = new DOMConfiguration( file );
		readInfo( config );

		generateDao();
		generateDto();
	}

	void readInfo( IConfiguration config ) throws Exception {
		IConfiguration c;
		c = config.getChild( "template" );
		if ( c != null ) {
			readTemplate( c );
		}
		c = config.getChild( "output" );
		if ( c != null ) {
			readOutput( c );
		}

		c = config.getChild( "table" );
		readTable( c );
		c = config.getChild( "access" );
		readAccess( c );

		if ( daoClass == null ) {
			daoClass = table.getName() + "Dao";
		}
		if ( dtoClass == null ) {
			dtoClass = table.getName();
		}

	}

	void readTemplate( IConfiguration config ) throws Exception {
		if ( template == null ) {
			template = new Template();
		}
		IConfiguration attr = config.getAttributes();
		template.setDao( attr.get( "dao" ));
		template.setDto( attr.get( "dto" ));
		template.setMethod( attr.get( "method" ));
	}

	void readOutput( IConfiguration config ) throws Exception {
		IConfiguration attr = config.getAttributes();
		daoClass = attr.get( "dao", null );
		dtoClass = attr.get( "dto", null );
	}

	void readTable( IConfiguration config ) throws Exception {
		table = new TableInfo( config );
		table.setTemplate( template );
	}

	void readAccess( IConfiguration config ) throws Exception {
		access = new AccessInfo( config );
		access.setTemplate( template );
	}

	void generateDao() throws Exception {
		StringBuffer buf = template.readTemplate( template.getDao() );
		template.replacePattern( buf, "<$$$ ClassName $$$>", daoClass );

		String method = access.getMethod( dtoClass );
		template.replacePattern( buf, "<$$$ Method $$$>", method );

		String fname = daoClass + ".java";
		if ( destdir != null ) {
			fname = destdir + File.separator + fname;
		}
		FileWriter out = new FileWriter( fname );
		String res = buf.toString();
		out.write( res, 0, res.length() );
		out.flush();
		out.close();
	}

	void generateDto() throws Exception {
		StringBuffer buf = template.readTemplate( template.getDto() );
		template.replacePattern( buf, "<$$$ ClassName $$$>", dtoClass );
		String field = table.getField();
		template.replacePattern( buf, "<$$$ Field $$$>", field );
		String method = table.getMethod();
		template.replacePattern( buf, "<$$$ Method $$$>", method );

		String fname = dtoClass + ".java";
		if ( destdir != null ) {
			fname = destdir + File.separator + fname;
		}
		FileWriter out = new FileWriter( fname );
		String res = buf.toString();
		out.write( res, 0, res.length() );
		out.flush();
		out.close();
	}

}
